local Emphasize = Grid:GetModule("GridStatus"):GetModule("GridStatusEmphasize")
local GridStatusEmphasize_Tanks = Emphasize:NewModule("GridStatusEmphasize_Tanks", "AceEvent-2.0")

local GridRoster = Grid:GetModule("GridRoster")

-- upvalues
local UnitInRaid = UnitInRaid
local UnitInParty = UnitInParty
local GetPartyAssignment = GetPartyAssignment
local UnitGroupRolesAssigned = UnitGroupRolesAssigned

GridStatusEmphasize_Tanks.defaultDB = {
	always_show_tanks = true,
}

local options = {
	["show_tanks"] = {
		type = "toggle",
		name = "Never fade out tanks",
		desc = "Always keep tanks visible, even during emphasizes",
		get = function () return GridStatusEmphasize_Tanks.db.profile.always_show_tanks end,
		set = function (v)
				GridStatusEmphasize_Tanks.db.profile.always_show_tanks = v
				GridStatusEmphasize_Tanks:UpdateMainTanks()	
			end,
	},
}

local settings
local zoning = false

function GridStatusEmphasize_Tanks:OnInitialize()
	self.super.OnInitialize(self)
	
	self:RegisterOptions("Tanks", options)
	settings = self.db.profile

	self:RegisterEvent("PLAYER_LOGIN", "UpdateMainTanks")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	
	self:EnableDisable()
end

-- workaround to stop "<unit> is not in your party" spam
function GridStatusEmphasize_Tanks:PLAYER_ENTERING_WORLD()
	zoning = false
	self:UpdateMainTanks()
	self:Debug("PLAYER_ENTERING_WORLD")
end

function GridStatusEmphasize_Tanks:PLAYER_LEAVING_WORLD()
	zoning = true
	self:ScheduleEvent("GSE_T_Force_PLAYER_ENTERING_WORLD", "PLAYER_ENTERING_WORLD", 5)
	self:Debug("PLAYER_LEAVING_WORLD")
end

function GridStatusEmphasize_Tanks:EnableDisable()
	if settings.always_show_tanks then
		self:RegisterEvent("PLAYER_ROLES_ASSIGNED", "UpdateMainTanks")
		self:RegisterEvent("LFG_ROLE_UPDATE", "UpdateMainTanks")
	elseif self:IsEventRegistered("PLAYER_ROLES_ASSIGNED") then
		self:UnregisterEvent("PLAYER_ROLES_ASSIGNED", "UpdateMainTanks")
		self:UnregisterEvent("LFG_ROLE_UPDATE", "UpdateMainTanks")
	end	

	self:UpdateMainTanks()
end

function GridStatusEmphasize_Tanks:UpdateMainTanks()
	self:Debug("Update tanks ", zoning)

	Emphasize:Pause()

	Emphasize:StaticDeemphasizeAllUnits("main_tank")

	if not zoning and settings.always_show_tanks and GridRoster:GetPartyState() ~= "solo" then
		for guid, unitid in GridRoster:IterateRoster() do
			if (UnitInParty(unitid) or UnitInRaid(unitid)) and (GetPartyAssignment("MAINTANK", unitid, true) or UnitGroupRolesAssigned(unitid) == true) then
				self:Debug(UnitName(unitid) .. " is tank - " .. UnitGUID(unitid) .. " - " .. guid)
				Emphasize:StaticEmphasizeUnit(guid, "main_tank")
			end
		end
	end

	Emphasize:Resume()	
end
